Module git_config::file[][src]

This module provides a high level wrapper around a single git-config file.

Structs

High level git-config reader and writer.

An intermediate representation of a mutable multivar obtained from GitConfig.

A opaque type that represents a mutable reference to a section.

An intermediate representation of a mutable value obtained from GitConfig.

A opaque type that represents a section body.

Enums

All possible error types that may occur from interacting with GitConfig.