Enum git_config::parser::Event[][src]

pub enum Event<'a> {
    Comment(ParsedComment<'a>),
    SectionHeader(ParsedSectionHeader<'a>),
    Key(Key<'a>),
    Value(Cow<'a, [u8]>),
    Newline(Cow<'a, str>),
    ValueNotDone(Cow<'a, [u8]>),
    ValueDone(Cow<'a, [u8]>),
    Whitespace(Cow<'a, str>),
    KeyValueSeparator,
}
Syntactic events that occurs in the config. Despite all these variants holding a Cow instead over a simple reference, the parser will only emit borrowed Cow variants.

The Cow smart pointer is used here for ease of inserting events in a middle of an Event iterator. This is used, for example, in the GitConfig struct when adding values.

Variants

Comment

A comment with a comment tag and the comment itself. Note that the comment itself may contain additional whitespace and comment markers at the beginning.

Tuple Fields of Comment

0: ParsedComment<'a>
SectionHeader

A section header containing the section name and a subsection, if it exists.

Tuple Fields of SectionHeader

0: ParsedSectionHeader<'a>
Key

A name to a value in a section.

Tuple Fields of Key

0: Key<'a>
Value

A completed value. This may be any string, including the empty string, if an implicit boolean value is used. Note that these values may contain spaces and any special character. This value is also unprocessed, so it it may contain double quotes that should be replaced.

Tuple Fields of Value

0: Cow<'a, [u8]>
Newline

Represents any token used to signify a new line character. On Unix platforms, this is typically just \n, but can be any valid newline sequence. Multiple newlines (such as \n\n) will be merged as a single newline event.

Tuple Fields of Newline

0: Cow<'a, str>
ValueNotDone

Any value that isn’t completed. This occurs when the value is continued onto the next line. A Newline event is guaranteed after, followed by either a ValueDone, a Whitespace, or another ValueNotDone.

Tuple Fields of ValueNotDone

0: Cow<'a, [u8]>
ValueDone

The last line of a value which was continued onto another line.

Tuple Fields of ValueDone

0: Cow<'a, [u8]>
Whitespace

A continuous section of insignificant whitespace. Values with internal spaces will not be separated by this event.

Tuple Fields of Whitespace

0: Cow<'a, str>
KeyValueSeparator

This event is emitted when the parser counters a valid = character separating the key and value. This event is necessary as it eliminates the ambiguity for whitespace events between a key and value event.

Implementations

Generates a byte representation of the value. This should be used when non-UTF-8 sequences are present or a UTF-8 representation can’t be guaranteed.

Coerces into an owned instance. This differs from the standard clone implementation as calling clone will not copy the borrowed variant, while this method will. In other words:

Borrow type.clone()to_owned()
BorrowedBorrowedOwned
OwnedOwnedOwned

This can be most effectively seen by the differing lifetimes between the two. This method guarantees a 'static lifetime, while clone does not.