Module git_config::values [−][src]
Rust containers for valid git-config types.
Structs
| Color | Any value that may contain a foreground color, background color, a collection of color (text) modifiers, or a combination of any of the aforementioned values. |
| Integer | Any value that can be interpreted as an integer. |
Enums
| Boolean | Any value that can be interpreted as a boolean. |
| ColorAttribute | Discriminating enum for |
| ColorParseError | Discriminating enum for |
| ColorValue | Discriminating enum for |
| IntegerSuffix | Integer prefixes that are supported by |
| TrueVariant | Discriminating enum between implicit and explicit truthy values. |
| Value | Fully enumerated valid types that a |
Functions
| normalize_bytes |
|
| normalize_cow | Removes quotes, if any, from the provided inputs. This assumes the input contains a even number of unescaped quotes, and will unescape escaped quotes. The return values should be safe for value interpretation. |
| normalize_str |
|
| normalize_vec |
|